part of tecfy_core_client_package;

class ListFormProperty extends StatefulWidget {
  final String entityId;
  final String? label;
  final String? description;
  final bool isRequired;
  final bool readOnly;
  final bool isAddNew;
  final List<dynamic> data;
  final void Function(List<dynamic>)? onChanged;
  ListFormProperty(this.isAddNew,
      {required this.entityId,
      this.label,
      this.description,
      this.isRequired = false,
      this.data = const [],
      this.onChanged,
      this.readOnly = false});
  @override
  _ListFormPropertyState createState() => _ListFormPropertyState();
}

class _ListFormPropertyState extends AppState<ListFormProperty> {
  Entity? _entity;
  late ScrollController _scrollController;

  _onValueChanged(value, propName, Map<String, dynamic> data) async {
    print('inline value change! $propName');
    data[propName] = value;

    bool updated = await _entity!.setDefaultValues(data);

    // for (Property p in entity.editProperties) {
    //   var r = await p.isVisble(data, entity);
    //   if (r != p.isVisable) {
    //     p.isVisable = r;
    updated = true;
    //   }
    // }

    // check the calced properties
    for (PropertyNumber p in _entity?.calcProps ?? []) {
      if (p.calcedProps.contains(p.upperFirst(propName)) ||
          p.calcedProps.contains(propName)) {
        var vars =
            p.calcedProps.map((pro) => data[p.lowerFirst(pro)] ?? 0).toList();
        data[p.dbName] = p.getCalcResult(vars);
        setState(() {});
      }
    }

    if (updated ||
        _entity!.editProperties.any((p) => p.filterValueField == propName))
      setState(() {});

    widget.onChanged?.call(widget.data);
  }

  _onUpdateData(Map<String, dynamic> data, Property property) {
    // we need this for Ramy's application to update the records when select an Item
    if (property.propertyType == PropertyTypes.Dropdown) {
      var prop = property as PropertyDropdown;
      if (data[prop.dbNameEntity] == null) return;
      var props = property.overwritePropertiesOnSelect?.keys ?? [];
      for (var key in props) {
        _onValueChanged(data[prop.dbNameEntity][key],
            property.overwritePropertiesOnSelect![key], data);
      }
      setState(() {});
    }
  }

  _scannBarcode() async {
    var result =
        await AppNavigator.showAppDialog(context, (c) => ScannerCameraDialog());

    if (result != null) {
      print(
          '-------------start get data for ($result) for [${_entity!.dbName}]');
      var service = TecfyCoreApp.entityServices[_entity!.dbName.toLowerCase()];
      var data = await service?.getDataByBarcode(context, result, {});
      print(data);
      if (data?.data != null) setState(() => widget.data.add(data!.data));
    }
  }

  @override
  void initState() {
    _scrollController = ScrollController();
    _entity = EntityService.getEntity(widget.entityId)!;
    if (_entity == null)
      print(
          'XXXXXXXXXXXX  Error Cannot find entity structure  ${widget.entityId}');
    super.initState();
  }

  @override
  void dispose() {
    _scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return LayoutService.bindWidget(context,
        mobileWidget: _getMobile,
        descktopWidget: _getDesktop,
        tabletWidget: _getMobile);
  }

  Widget _getDesktop() {
    return Column(children: [
      Scrollbar(
          isAlwaysShown: true,
          showTrackOnHover: true,
          controller: _scrollController,
          child: SingleChildScrollView(
            controller: _scrollController,
            scrollDirection: Axis.horizontal,
            child: DataTable(
              showCheckboxColumn: true,
              columns: _entity!.editProperties
                  .map<DataColumn>((p) => DataColumn(
                      label: Text(p.name),
                      numeric: p.valueType == ValueTypes.Number,
                      tooltip: p.nameEn))
                  .toList()
                ..add(DataColumn(label: Text(''))), // for remove button
              rows: widget.data
                  .map<DataRow>(
                    (data) => DataRow(
                      cells: _entity!.editProperties
                          .map<DataCell>(
                            (p) => DataCell(
                              PorpertyComponent(
                                readOnly: widget.readOnly,
                                noLabel: true,
                                isAddNew: widget.isAddNew,
                                property: p,
                                data: data,
                                entity: _entity!,
                                onChange: (v) =>
                                    _onValueChanged(v, p.dbName, data),
                                onUpdateData: (v) => _onUpdateData(v, p),
                              ),
                            ),
                          )
                          .toList()
                        ..add(DataCell(widget.readOnly
                            ? SizedBox()
                            : IconButton(
                                icon: Icon(Icons.close),
                                tooltip: trans('Remove'),
                                onPressed: widget.readOnly
                                    ? null
                                    : () => setState(
                                        () => widget.data.remove(data))))),
                    ),
                  )
                  .toList(),
            ),
          )),
      SizedBox(height: 30),
      widget.readOnly
          ? Container()
          : SizedBox(
              width: 250,
              child: AppButtonIconText(
                label: '${trans('Click to add new')} ${widget.label}',
                icon: Icon(Icons.add, color: Colors.white),
                onPressed: () async =>
                    setState(() => widget.data.add(Map<String, dynamic>())),
              ),
            ),
    ]);
  }

  Widget _getMobile() {
    return Column(
      children: widget.data
          .map<Widget>((item) => InLineForm(widget.entityId, widget.isAddNew,
              readOnly: widget.readOnly,
              entity: _entity,
              data: item,
              onChanged: (v) => widget.onChanged?.call(widget.data),
              onDismiss: (v) => setState(() => widget.data.remove(v))))
          .toList()
        ..add(
          widget.readOnly
              ? Container()
              : Row(mainAxisAlignment: MainAxisAlignment.center, children: [
                  Container(
                    //width: 250,
                    constraints: BoxConstraints(maxWidth: 150),
                    child: AppButtonIconText(
                      isFullRow: true,
                      label: '${trans('Click to add new')} ${widget.label}',
                      icon: Icon(Icons.add, color: Colors.white),
                      onPressed: () async => setState(
                          () => widget.data.add(Map<String, dynamic>())),
                    ),
                  ),
                  SizedBox(width: 10),
                  _entity!.hasBarcode
                      ? IconButton(
                          icon: Icon(Icons.qr_code), onPressed: _scannBarcode)
                      : Container()
                ]),
        ),
    );
  }
}
